/***********************************************************************\
 *
 *	Copyright 1995-1997, Dolphin Safe Software DBA Pacific Coast Software
 *
 *  mod_webcatalog.cpp
 *
 *	This is the cgi client program which uses cgi conventions
 *  to serve the web-users
 *	
 *
 *
\***********************************************************************/

extern "C" {
#include <ctype.h>                  // for isprint()
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <sys/socket.h>

#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"
#include "http_request.h"
#include "util_script.h"
#include "apr_strings.h"
#include "apr_lib.h"
}

#include "wc_globals.h"

#define REMOTEADDRESS_SIZE    100
#define REMOTEUSER_SIZE       256
#define REMOTEPASSWORD_SIZE   100
//#define DEBUG 1

void ExpandIPAddress( char *theAddress );
void WriteDebug( const char *text, char *altText );
void WriteDebugN( const char *text, int num );
void DisplayLastError();;
void InternalError( char *text );
static int get_request_info(request_rec *req, char **buf);
int DecodePW(char *http_authorization,char *user,char *password);
void Base64Decode( char *bufcoded, unsigned char *bufplain, int outbufsize );
static void webcatalog_hooks(apr_pool_t *p);


// DecodePW(char *http__authorization,char *user,char *password)
//
// if http_authorization contains a basic base64 encoded user:password
// DecodePW will decode the user and password and put it in the appropriate
// variables
//
// returns 1 if successfull
// returns 0 otherwise
//
// note HTTP_AUTHORIZATION comes from authorization: basic <base 64encoded user:password>

int DecodePW(char *http_authorization,char *user,char *password)
{
	char decoded[100];
	char encoded[100];
	char *d;


	// initialize the password and user at ""
	*password=*user=0;

	// return if http_authorization is a null pointer
	if (!http_authorization) return 0;

	// copy to encoded
	strcpy(encoded,http_authorization);

	// get the first 5 characters
	encoded[5]=0;

	// check to see if it's basic encoding
	if (strcmp(encoded,"Basic")) return 0;

	// decode
	Base64Decode(encoded+6,(unsigned char *)decoded,99);


	// put d at first char 
	d=decoded;


	// get user
	while(*d!=0 && *d!=':')
	{
		*user++=*d++;
	}
	*user=0;
	if (d==0) 
	{
		*password=0;
		return 1;
	}
	d++;

	// get password
	while(*d)
	{
		*password++=*d++;
	}
	*password=0;
	return 1;
}

/*
 * Define translation matrix for Base64 decode.
 * it's fast and const should make it shared text page.
 */
const int pr2six[256] =
{
	64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
	64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,62,64,64,64,63,
	52,53,54,55,56,57,58,59,60,61,64,64,64,64,64,64,64,0,1,2,3,4,5,6,7,8,9,
	10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,64,64,64,64,64,64,26,27,
	28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,
	64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
	64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
	64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
	64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
	64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,
	64,64,64,64,64,64,64,64,64,64,64,64,64
};

/*
 * Decode a base64 encoded string
 */
void Base64Decode(
	char *bufcoded,
	unsigned char *bufplain,
	int outbufsize
)
{
	int                         nbytesdecoded;
	int                         nprbytes;
	char                        *bufin = bufcoded;
	unsigned char       *bufout = bufplain;

	// Strip leading whitespace
	while (*bufcoded == ' ' || *bufcoded == '\t')
		++bufcoded;

	//
	//  Figure out how many characters are in the input buffer.
	//  If this would decode into more bytes than would fit into
	//  the output buffer, adjust the number of input bytes downwards.
	// 
	bufin = bufcoded;
	while (pr2six[*(bufin++)] <= 63)
		;
	nprbytes = bufin - bufcoded - 1;
	nbytesdecoded = ((nprbytes + 3) / 4) * 3;
	if (nbytesdecoded > outbufsize)
		nprbytes = (outbufsize * 4) / 3;

	bufin = bufcoded;
	while (nprbytes > 0)
	{
		*(bufout++) =
			(unsigned char) (pr2six[*bufin] << 2 | pr2six[bufin[1]] >>4);
		*(bufout++) =
			(unsigned char) (pr2six[bufin[1]] << 4 | pr2six[bufin[2]] >>2);
		*(bufout++) =
			(unsigned char) (pr2six[bufin[2]] << 6 | pr2six[bufin[3]]);
		bufin += 4;
		nprbytes -= 4;
	}

	if(nprbytes & 03)
	{
		if(pr2six[bufin[-2]] > 63)
			nbytesdecoded -= 2;
		else
			nbytesdecoded -= 1;
	}
	bufplain[nbytesdecoded] = '\0';
}

void
ERROR(request_rec* r, char* s)
{
}

void cleanup(request_rec* r)
{
}

void
cleanExit(request_rec* r, int exitCode)
{
	//WriteDebugN("In cleanExit: ", exitCode);
	cleanup(r);
	exit(exitCode);
}

void
DIE(request_rec* r, char* s)
{
	//WriteDebug("In DIE: ", s);
	ERROR(r, s);
	cleanExit(r, 1);
}


// char *strecpy(char *dest,char *source)
// just a normal strcpy, but if source==0 then dest will be an empty string
char *strecpy(char *dest,char *source)
{
	if (source)
	{
		strcpy(dest,source);
		return dest;
	}
	dest[0]=0;
	return dest;
}

void
ExpandIPAddress( char *theAddress )
{
	if( isdigit( *theAddress ) )
	{
		int num1, num2, num3, num4;
		sscanf( theAddress, "%d.%d.%d.%d", &num1, &num2, &num3, &num4 );
		sprintf( theAddress, "%.3d.%.3d.%.3d.%.3d", num1, num2, num3, num4 );
	}
}


void WriteDebugN( const char *text, int num )
{
	char buf[16];
	sprintf(buf, "%d", num);
	WriteDebug(text, buf);
}

void WriteDebug( const char *text, char *altText )
{
	static	FILE	*fp = NULL;
#ifdef DEBUG
	if( !fp )
	{
		fp = fopen( "/tmp/workermaindebug.txt", "a" );
		if( !fp )
		{
			InternalError("fp=NULL\n");
			perror("/tmp/workermaindebug.txt");
			return;
		}
	}
	fseek( fp, SEEK_END, 0L );
	fprintf( fp, "%.3d:", getpid() );
	fprintf( fp, "%s\t%s\n", text, altText );
	fflush( fp );
#endif
}

void InternalError( char *text )
{
#ifdef DEBUG
	FILE	*theFile = fopen( "UNIXCGIDebug.txt", "a" );
	if( theFile )
	{
		fprintf( theFile, text );
		fclose( theFile );
	}
#endif
}


static char *lookup_header(request_rec *r, const char *name)
{
    apr_array_header_t *hdrs_arr;
    apr_table_entry_t *hdrs;
    int i;

    hdrs_arr = (apr_array_header_t *)apr_table_elts(r->headers_in);
    hdrs = (apr_table_entry_t *)hdrs_arr->elts;
    for (i = 0; i < hdrs_arr->nelts; ++i) {
        if (hdrs[i].key == NULL) {
            continue;
        }
        if (strcasecmp(hdrs[i].key, name) == 0) {
	    return hdrs[i].val;
        }
    }
    return NULL;
}

static int webcatalog_handler(request_rec *r)
{
	int sockfd;			// socket file descriptor
#if defined(_SOLARISCGI) || defined(_MACOS10CGI)
	size_t len;			// length of the socket structure
#else
	socklen_t len;			// length of the socket structure
#endif
	struct sockaddr_un addr;	// socket info
	char *buffer = NULL;		// contains the request info
	unsigned int buf_size = 0;		// the size of the send buffer (Using int here so that 32&64 bit apps can work together)
	char  *result_page = NULL;	// result page received from WebCatalog


	// Always process pages that register with AddHandler directive
	if (strcmp(r->handler, "webcatalog2-handler") ) {
		return DECLINED;
	}

	//
	// Try to establish a connection to the WebCatalog server.
	//
	sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
	bzero(&addr, sizeof(addr));
	addr.sun_family = AF_UNIX;
	strncpy(addr.sun_path, "/tmp/.webcatalog", sizeof(addr.sun_path) - 1);
	//ap_set_content_type(r,"text/html");

	//
	// If the connection fails we assume that WebCatalog is not
	// running so we send an appropriate page to Apache.
	//
	if (connect(sockfd, (struct sockaddr*) &addr, sizeof(addr)) != 0)
	{
		r->status = 503;
		ap_rputs("<html><body>Sorry WebDNA server not running.</body></html>", r);
		close(sockfd);
		return OK;
	}
   

	//
	// Get all request data needed by WebCatalog.
	//
	buf_size = get_request_info(r, &buffer);

	// 
	// Send the data to the server.
	//
	socket_write(sockfd, &buf_size, sizeof(buf_size));
	socket_read(sockfd, &buf_size, sizeof(buf_size)); 
	socket_write(sockfd, buffer, buf_size);
   
	//
	// Send the size of the buffer we are about to
	// send to WebCatalog and get the responce.
	//
	socket_read(sockfd, &buf_size, sizeof(buf_size));
	socket_write(sockfd, &buf_size, sizeof(buf_size));
   
	//
	// Allocate enough memory for the result.
	//
	result_page = (char *)apr_palloc(r->pool, buf_size);
   
WriteDebugN("buf_size",buf_size);
	//
	// Read the result header and body from WebCatalog.
	//
	socket_read(sockfd, result_page, buf_size);

        // Parse out the headers
        char* header_page = result_page;
	int nCookies = 0;
	int header_len = 0;

        result_page = strstr(result_page, "\r\n\r\n") + 4;
        *(result_page-4) = 0;

	header_len = result_page - header_page;
WriteDebugN("header_len",header_len);
        // Skip over the 'result' line
        char* szTemp = strstr(header_page, "\r\n");
        *szTemp = 0;
        szTemp += 2;

        // Loop through the headers returned from our engine and send each one to Apache
        while (*szTemp)
        {
            // get MIME name
            char* szName = szTemp;
            while (*szTemp != ':') szTemp++;

            *szTemp = 0;
            szTemp += 2;

            //get MIME value

            //get MIME value
            char* szValue = szTemp;
            while ((*szTemp != '\r' && *(szTemp + 1) != '\n') && *szTemp != 0) szTemp++;

            if (*szTemp != 0)
            {
                *szTemp = 0;
                szTemp += 2;
            }

            //
            //
            // Ignore the Content-Length, Server, and Date headers
            //
            if (!strstr("Content-Length Server Date ", szName))
            {
                apr_table_add(r->headers_out, szName, szValue);
            }
        }
        if (strstr(header_page, "401"))
            r->status = 401;
        if (strstr(header_page, "302"))
            r->status = 302;
        if (strstr(header_page, "404"))
            r->status = 404;
        if (strstr(header_page, "200"))
            r->status = 200;

        //
        // Send the resulting page to Apache
	//r->sent_bodyct = ap_rputs(result_page, r);
WriteDebugN("result_len",buf_size-header_len);
	r->sent_bodyct = ap_rwrite(result_page, buf_size - header_len, r);

	//
	// Free all memory allocated and
	// close the socket connection.
	//
	free(buffer);
	close(sockfd);

	//
	// We did what we wanted to do, so tell 
	// the rest of the server we succeeded.
	//
	cleanup(r);

	return OK;
}


/* Obtain the Request-URI from the original request-line, returning
 * a new string from the request pool containing the URI or "".
 */
static char *original_uri(request_rec *r)
{
    char *first, *last;

    if (r->the_request == NULL) {
        return (char *) apr_pcalloc(r->pool, 1);
    }
    first = r->the_request;     /* use the request-line */

    while (*first && !apr_isspace(*first)) {
        ++first;                /* skip over the method */
    }
    while (apr_isspace(*first)) {
        ++first;                /*   and the space(s)   */
    }

    last = first;
    //while (*last && !apr_isspace(*last)) {
    while (*last && !apr_isspace(*last) && *last != '?' ) {
        ++last;                 /* end at next whitespace */
    }

    return apr_pstrndup(r->pool, first, last - first);
}

/* Obtain the Request-query string from the original request-line, returning
 * a new string from the request pool containing the URI or "".
 */
static char *original_query(request_rec *r)
{
    char *first, *last;

    if (r->the_request == NULL) {
        return (char *) apr_pcalloc(r->pool, 1);
    }
    first = r->the_request;     /* use the request-line */

    while (*first && *first != '?') {
        ++first;                /* skip over the method & URL */
    }

    if (*first == '?') {
        ++first;                /*   and the space(s)   */
    }

    last = first;
    while (*last && !apr_isspace(*last) ) {
        ++last;                 /* end at next whitespace */
    }

    return apr_pstrndup(r->pool, first, last - first);
}



//*****************************************************************************
// Function:
//    get_request_info
//
// Description:
//    This function 
//
// Arguments:
//    req      (in) - Pointer to the request record.
//    buff (in/out) - Pointer to the buffer where the data is stored.
//
// Returns: 
//   -1 - If buf is NULL.
//    1 - Otherwise
//
//*****************************************************************************
static int get_request_info(
   request_rec *req,
   char **buf
)
{
	apr_array_header_t	*aah = (apr_array_header_t *)apr_table_elts(req->headers_in);
	apr_table_entry_t	*elts = (apr_table_entry_t *) aah->elts;
	char	*dynamic_buf = NULL;
	char	*cur_ptr = NULL;
	size_t	buf_size = 0;
	char 	*user_agent = NULL;
	char	remote_addr[REMOTEADDRESS_SIZE] = "";
	char	username[REMOTEUSER_SIZE] = "";
	char	remotepassword[REMOTEPASSWORD_SIZE] = "";
	char	base64buf[200] = "";
	char	*postArgData = NULL, *tempPtr;
	int	total_bytes=0;
	char	postArgsLen[80];
	char	orgURL[256];

	//
	// Make sure buf is not NULL.
	//
	if (!buf)
	{
		return (-1);
	}

	//
	// Get POST data from the request record.
	//
	ap_setup_client_block(req, REQUEST_CHUNKED_DECHUNK);
	if (ap_should_client_block(req)) 
	{
		void (*handler) (int);
		int len_read;
		char argsbuffer[HUGE_STRING_LEN];

		handler = signal(SIGPIPE, SIG_IGN);

		while ((len_read = ap_get_client_block(req, argsbuffer, HUGE_STRING_LEN)) > 0) 
		{
			postArgData = (char*) realloc(postArgData, total_bytes + len_read + 1);
			memcpy(postArgData + total_bytes, argsbuffer, len_read);
			total_bytes += len_read;
			postArgData[total_bytes] = 0;
		}
		signal(SIGPIPE, handler);
	}

	// Get the method type.
	write_string(&dynamic_buf, &cur_ptr, (char *)req->method, buf_size);
	
	// Get the server name.
	write_string(&dynamic_buf, &cur_ptr, 
			(char *)ap_get_server_version(), buf_size);

	// Get the document root.
	write_string(&dynamic_buf, &cur_ptr, 
			(char *)ap_document_root(req), buf_size);

	// Get the requested arguments.
	if (strcmp(req->method, "GET") == 0)
	{
		// Get QUERY_STRING
		if (req->status != HTTP_NOT_FOUND)
			write_string(&dynamic_buf, &cur_ptr, req->args, buf_size);
		else 
			write_string(&dynamic_buf, &cur_ptr, original_query(req), buf_size);
	}
	else
	{
		// Check for QUERY data
		if (req->args)
		{
			int nTemp = strlen(req->args);
			postArgData = (char*) realloc(postArgData, total_bytes + nTemp + 2);
			postArgData[total_bytes] = '&';
			total_bytes += 1;
			memcpy(postArgData + total_bytes, req->args, nTemp);
			total_bytes += nTemp;
			postArgData[total_bytes] = 0;
		}
		sprintf(postArgsLen, "%d", total_bytes);
		write_string(&dynamic_buf, &cur_ptr, postArgsLen, buf_size);
   
		if (total_bytes)
		{
			write_mem(&dynamic_buf, &cur_ptr, postArgData, total_bytes, buf_size);
		}
	}
	
	//	Get URL
	if (req->status != HTTP_NOT_FOUND)
	{
		// Code from main/util_script.c  I don't understand it,
		// but apparently there's special magic to determine
		// what the SCRIPT_NAME is - SLG
		if ((!strcmp(req->protocol, "INCLUDED"))  
		 || (!req->path_info || !*req->path_info)) 
		{
			write_string(&dynamic_buf, &cur_ptr, req->uri, buf_size);
		} 
		else 
		{
			int path_info_start = ap_find_path_info(req->uri, req->path_info);
			write_string(&dynamic_buf, &cur_ptr, 
				apr_pstrndup(req->pool, req->uri, path_info_start), buf_size);
		}
	}
	else
	{
		write_string(&dynamic_buf, &cur_ptr, original_uri(req), buf_size);
	}

	// Get the script name.
	if (req->status != HTTP_NOT_FOUND)
		write_string(&dynamic_buf, &cur_ptr, req->filename, buf_size);
	else
	{
		strcpy(orgURL,(char *)ap_document_root(req));
		strcat(orgURL,original_uri(req));
		write_string(&dynamic_buf, &cur_ptr, orgURL, buf_size);
	}	

	//
	// Get HTTP_USER_AGENT
	//
	user_agent = lookup_header(req, "User-Agent");
	if(!user_agent)
	{
		user_agent = "Unknown";
	}
	write_string(&dynamic_buf, &cur_ptr, user_agent, buf_size);
	
	// Get the remote address.
	if (lookup_header(req, "PC-Remote-Addr"))
	{
		strcpy(remote_addr, lookup_header(req, "PC-Remote-Addr") );
	}
	else
	{
		strcpy(remote_addr, req->connection->remote_ip);
	}
	ExpandIPAddress(remote_addr);
	write_string(&dynamic_buf, &cur_ptr, remote_addr, buf_size);


	strcpy(base64buf, "");
	strcpy(remotepassword, "");

	// Netscape and MIIS
	// have REMOTE_USER
	if (req->user)
	{
		strcpy(username, req->user);
	}
	
	if (lookup_header(req, "Authorization"))
	{
		strcpy(base64buf, lookup_header(req, "Authorization") );
	}
	if( base64buf[0] )
		DecodePW(base64buf, username, remotepassword);

	write_string(&dynamic_buf, &cur_ptr, username, buf_size);
	write_string(&dynamic_buf, &cur_ptr, remotepassword, buf_size);

	// Get the referer string.
	write_string(&dynamic_buf, &cur_ptr, lookup_header(req, "Referer"), buf_size);
	
	write_string(&dynamic_buf, &cur_ptr,
			(char *)apr_table_get(req->headers_in, "Cookie"), buf_size);

	//	Add MIME header stuff to params
	// Write incoming MIME headers
	for (int i = 0; i < aah->nelts; i++) 
	{
		if (i == 0)
		{
			write_string(&dynamic_buf, &cur_ptr, (char*)elts[i].key, buf_size);
		}
		else
		{
			append_to_last_string(&dynamic_buf, 
				&cur_ptr, (char*)elts[i].key, buf_size);
		}
		append_to_last_string(&dynamic_buf, &cur_ptr, ":", buf_size);
		append_to_last_string(&dynamic_buf, &cur_ptr, 
				(char*)elts[i].val, buf_size);
		append_to_last_string(&dynamic_buf, &cur_ptr, "\r\n", buf_size);
	}

	write_string(&dynamic_buf, &cur_ptr, "", buf_size);
   
	free(postArgData);

	*buf = dynamic_buf;
   
	return (cur_ptr - dynamic_buf);

} // end get_request_info


static void register_hooks(apr_pool_t *p)
{
	ap_hook_handler(webcatalog_handler,NULL,NULL, APR_HOOK_MIDDLE);
}

module AP_MODULE_DECLARE_DATA webcatalog2_module =
{
    STANDARD20_MODULE_STUFF,
    NULL,			/* per-directory config creator */
    NULL,			/* dir config merger */
    NULL,			/* server config creator */
    NULL,			/* server config merger */
    NULL,			/* command table */
    register_hooks,		/* register webcatalog2_handler */
};
