#!/bin/bash
#
# Usage: install.sh 
# This script installs the WebCatalog packet under Apache directories

ECHO=/bin/echo
CP=/bin/cp
MV=/bin/mv
RM=/bin/rm
TAR=/bin/tar
SED=`which sed`
CHOWN=/bin/chown
CHGRP=/bin/chgrp
CHMOD=/bin/chmod
MKDIR=/bin/mkdir
DIRNAME=dirname

if [ `id | ${SED} -e 's/[^(]*(//' -e 's/).*//'` != root ]; then
      echo " "
      echo "ERROR: You must be root or sudo to install software."
      echo " "
      exit 1
fi

clear
$ECHO "==========================================================="
$ECHO "	START INSTALLATION					 "
$ECHO "==========================================================="
$ECHO ""
$ECHO "Welcome to the WebDNA Installer"
$ECHO ""
$ECHO "==========================================================="
$ECHO " LICENSE                                                   "
$ECHO "==========================================================="
$ECHO "To begin installation, please read and agree to the license"
$ECHO "agreement."
$ECHO "Press Enter to see the license agreement, and press space"
$ECHO "to advance through the pages."
$ECHO "After reviewing the licence agreement, press q to exit."
read WAIT_FOR_ENTER

#Show the License using more or cat.  Do not use less, because the user may not know to hit q
MORE_GUESS_ARRAY=( 'more' 'cat' )
MORE_BIN=`./try_until_success.sh which ${MORE_GUESS_ARRAY[@]}`
MORE_BIN=`which ${MORE_BIN}`
if [ $? != 0 ] || [ ! -x ${MORE_BIN} ]; then
  MORE_BIN=`which more`
fi

if [ $? -ne 0 ] || [ .${MORE_BIN} = . ]; then
  echo "The binary or alias to 'more' is not in your path"
  echo "It is required to read the license agreement."
  echo "Please install it and rerun this installer."
  exit 1
fi
if [ ! -f "./WebCatalog/End-User License Agreement.txt" ]; then 
  echo "Problems with installer files.  Please uncompress the directory first or redownload and uncompress fully."
  echo "Try:"
  echo "tar xzf WebDNA*.tar.gz"
  echo "Or:"
  echo "tar xzf WebDNA*.tar"
  exit 1
fi
${MORE_BIN} "./WebCatalog/End-User License Agreement.txt"

$ECHO "==========================================================="
$ECHO ""
LICENSE_QUESTION="Do you agree to the terms of the License Agreement? (Yes/No)"
$ECHO ${LICENSE_QUESTION}
while read AGREE_TO_LICENSE; do
  if [ .${AGREE_TO_LICENSE} = . ]; then 
    continue
  fi
  if [ ${AGREE_TO_LICENSE} = "Yes" ] || [ ${AGREE_TO_LICENSE} = "YES" ] || [ ${AGREE_TO_LICENSE} = "yes" ]; then
    break
  fi
  if [ ${AGREE_TO_LICENSE} = "No" ] || [ ${AGREE_TO_LICENSE} = "NO" ] || [ ${AGREE_TO_LICENSE} = "no" ]; then
    exit 1
  fi

  echo ${LICENSE_QUESTION}
done

if [ ${AGREE_TO_LICENSE} != "Yes" ] && [ ${AGREE_TO_LICENSE} != "YES" ] && [ ${AGREE_TO_LICENSE} != "yes" ]; then
  exit 1
fi

clear
$ECHO "==========================================================="
$ECHO " INSTALL                                                   "
$ECHO "==========================================================="
$ECHO "To install the WebDNA into your system, you will need "
$ECHO "to provide some infomation about your Apache Server."
$ECHO "Please be prepared to provide the paths for the following"
$ECHO "directories: "
$ECHO " Default Web Server DocumentRoot directory."
$ECHO " Default Web Server CGI directory."
$ECHO " Apache configuration directory (contains the default httpd.conf file)"
$ECHO " Apache user name"
$ECHO "Enter to accept the defaults"

if [ -d '/Library/Tenon/' ]; then
  $ECHO "==========================================================="
  $ECHO " iTools Detected"
  $ECHO "==========================================================="
  $ECHO "Installer defaults will be based on iTools default paths"
  $ECHO "If this is not your intention, please put in your preferred"
  $ECHO "paths in the following questions"
fi

#
# Get the path to the document root.
#
HTML_DIR_GUESS_ARRAY=( '/var/html/' '/srv/www/htdocs/' '/var/www/html/' '/var/www/' '/Library/Tenon/WebServer/Documents' '/Library/WebServer/Documents' )
HTML_DIR_GUESS=`./try_until_success.sh ls ${HTML_DIR_GUESS_ARRAY[@]]}`
if [ $? != 0 ] || [ ! -d ${HTML_DIR_GUESS} ]; then
  HTML_DIR_GUESS='/var/www/html'
fi
$ECHO ""
$ECHO " Enter the Default Web Server DocumentRoot directory:                "
$ECHO " ( default is ${HTML_DIR_GUESS} )"
$ECHO -n " "
read HTML

if [ .$HTML = . ]; then
   HTML=${HTML_DIR_GUESS}
fi
APACHE_DIR=`${DIRNAME} ${HTML}`
CHOSEN_APACHE_DIR_IS_TENON=`echo ${APACHE_DIR} | grep "/Library/Tenon"`
if [ -n "${CHOSEN_APACHE_DIR_IS_TENON}" ]; then
  TENON_BASE_DIR="/Library/Tenon/"
fi

if [ -d ${HTML}/WebCatalog/ ]; then
  $ECHO "==========================================================="
  $ECHO ""
  WEBCAT_QUESTION="${HTML}/WebCatalog/ will be deleted.  Are you sure you have a backup? (Yes/No)"
  $ECHO ${WEBCAT_QUESTION}
  while read AGREE_TO_WEBCAT_SAVE; do
    if [ .${AGREE_TO_WEBCAT_SAVE} = . ]; then 
      continue
    fi
    if [ ${AGREE_TO_WEBCAT_SAVE} = "Yes" ] || [ ${AGREE_TO_WEBCAT_SAVE} = "YES" ] || [ ${AGREE_TO_WEBCAT_SAVE} = "yes" ]; then
      break
    fi
    if [ ${AGREE_TO_WEBCAT_SAVE} = "No" ] || [ ${AGREE_TO_WEBCAT_SAVE} = "NO" ] || [ ${AGREE_TO_WEBCAT_SAVE} = "no" ]; then
      exit 1
    fi

    echo ${WEBCAT_QUESTION}
  done

  if [ ${AGREE_TO_WEBCAT_SAVE} != "Yes" ] && [ ${AGREE_TO_WEBCAT_SAVE} != "YES" ] && [ ${AGREE_TO_WEBCAT_SAVE} != "yes" ]; then
    exit 1
  fi
fi

#
# Get the path to the cgi-bin directory.
#
CGI_DIR_GUESS_ARRAY=( '${APACHE_DIR}/cgi-bin/' '/var/www/cgi-bin/' '/srv/www/cgi-bin/' '/usr/lib/cgi-bin/' '/Library/Tenon/WebServer/CGI-Executables/' '/Library/WebServer/CGI-Executables' )
CGI_DIR_GUESS=`./try_until_success.sh ls ${CGI_DIR_GUESS_ARRAY[@]}`
if [ $? != 0 ] || [ ! -d ${CGI_DIR_GUESS} ]; then
  CGI_DIR_GUESS='/var/www/cgi-bin'
fi
$ECHO ""
$ECHO " Enter the Default Web Server CGI directory:                "
$ECHO " ( default is ${CGI_DIR_GUESS} )"
$ECHO -n " "
read CGI_BIN

if [ .$CGI_BIN = . ]; then
   CGI_BIN=${CGI_DIR_GUESS}
fi
ENGINE_DIR=${CGI_BIN}/WebCatalogEngine

#
#Get the path to the httpd.conf file.
#
HTTPD_DIR_GUESS_ARRAY=( '/Library/Tenon/WebServer/Configuration'  '/etc/apache2' '/etc/httpd/conf' '/etc/httpd' )
HTTPD_DIR_GUESS=`./try_until_success.sh ls ${HTTPD_DIR_GUESS_ARRAY[@]}`
if [ $? != 0 ] || [ ! -d ${HTTPD_DIR_GUESS} ]; then
  HTTPD_DIR_GUESS='/etc/httpd'
fi

$ECHO ""
$ECHO " Enter the Apache configuration directory (contains the default httpd.conf file):             "
$ECHO " ( default is ${HTTPD_DIR_GUESS} )"
$ECHO -n " "
read  HTTPD

if [ .$HTTPD = . ]; then
   HTTPD=${HTTPD_DIR_GUESS}
fi

#
#Get the apache user
#
APACHE_USER_GUESS_ARRAY=( 'wwwrun' 'apache' 'www-data' '_www' 'www' )
WWW_USER_GUESS=`./try_until_success.sh id ${APACHE_USER_GUESS_ARRAY[@]}`
if [ $? != 0 ]; then
  WWW_USER_GUESS='apache'
fi
$ECHO ""
$ECHO " Enter the apache user name:             "
$ECHO " ( default is ${WWW_USER_GUESS} )"
$ECHO -n " "
read  WWW_USER

if [ .${WWW_USER} = . ]; then
   WWW_USER=${WWW_USER_GUESS}
fi
WWW_GROUP=`groups ${WWW_USER} | awk '{print $3}'`

#Fix libssl link issues
./fix_libssl.sh

#
# Determine if 32 or 64bit
#
MACH_HW=`uname -m`
MACH_BITS="i386"
LIB_DIR="/usr/lib"
case "${MACH_HW}" in
  x86_64)
    MACH_BITS="x86_64"
    LIB_DIR="/usr/lib64"
    ;;
  *)
    MACH_BITS="i386"
    LIB_DIR="/usr/lib"
    ;;
esac

case `uname -s` in 
  Darwin)
    WD_ARCH="macosx"
    MACH_BITS="i386"
    LIB_DIR="/usr/lib"
    ;;
  *)
    #
    # Determine glibc compatibility
    #
    WD_ARCH="glibc_3"
    TEST_EXE=WebCatalogEngine/${WD_ARCH}/${MACH_BITS}/WebCatalog
    IS_WD_ARCH_OK=`ldd ${TEST_EXE} 2>&1 |grep -v "xerces" | grep -v "xalan" | grep -i "not found"`
    if [ ."${IS_WD_ARCH_OK}" != . ]; then
      WD_ARCH="glibc_2"
      TEST_EXE=WebCatalogEngine/${WD_ARCH}/${MACH_BITS}/WebCatalog
      IS_WD_ARCH_OK=`ldd ${TEST_EXE} 2>&1 |grep -v "xerces" | grep -v "xalan"| grep -i "not found"`
    fi
    if [ ."${IS_WD_ARCH_OK}" != . ]; then
      $ECHO "Missing Library"
      $ECHO "Please install required libraries for your system. (Apache, mysqlclient)"
      exit 1
    fi
    $ECHO "Using WD_ARCH=${WD_ARCH}"
    ;;
esac

echo "Installing WebDNA..."

./create_dirs.sh ${HTML} ${ENGINE_DIR} ${WWW_USER}

APACHE_CTL_GUESS_ARRAY=( 'apache2ctl' 'apachectl' '/etc/init.d/apache' '/etc/init.d/apache2' '/etc/init.d/httpd' )
APACHE_CTL=`./try_until_success.sh which ${APACHE_CTL_GUESS_ARRAY[@]}`
echo "Using ApacheCtl as: ${APACHE_CTL}"

sudo ${APACHE_CTL} stop
sleep 2


##Copy WebCatalog HTML files
rm -rf ${HTML}/WebCatalog
mkdir ${HTML}/WebCatalog
$CP -r -p WebCatalog/* ${HTML}/WebCatalog/
$ECHO " Installed WebCatalog documents into directory:"
$ECHO " ${HTML}/WebCatalog  "
$CP -r -p WebCatalogEngine/* ${ENGINE_DIR}/
$CP -p ${ENGINE_DIR}/${WD_ARCH}/${MACH_BITS}/* ${ENGINE_DIR}/
rm -rf ${ENGINE_DIR}/glibc_3
rm -rf ${ENGINE_DIR}/glibc_2
rm -rf ${ENGINE_DIR}/macosx
$ECHO " Installed WebCatalog into directory"
$ECHO " ${ENGINE_DIR}  "
$CP LibXML/${WD_ARCH}/${MACH_BITS}/lib* ${LIB_DIR}
if [ "$WD_ARCH" = "glibc_2" ]; then
  #Redhat does not follow standard library location conventions, so cp libs to /usr/lib as well.
  $CP LibXML/${WD_ARCH}/${MACH_BITS}/lib* /usr/lib
fi
if [ ${WD_ARCH} != "macosx" ]; then
  ldconfig
fi
if [ ${WD_ARCH} = "macosx" ]; then
  chown root:wheel /System/Library/LaunchDaemons/org.apache.httpd.plist
  MACSTARTUPDIR=/Library/StartupItems/WebDNA/
  mkdir -p ${MACSTARTUPDIR}
  cp macosxstartup/* ${MACSTARTUPDIR}
  chmod -R 755 ${MACSTARTUPDIR}
  sed -i -e "s%/Library/WebServer/CGI-Executables/WebCatalogEngine%${ENGINE_DIR}%" ${MACSTARTUPDIR}/WebDNA
fi

if [ -n "${TENON_BASE_DIR}" ]; then
  #Copy itools files and update Register.pm
  ITOOLS_BIN=${TENON_BASE_DIR}/AdminServer/bin/
  ITOOLS_REGISTER_PM=${TENON_BASE_DIR}/AdminServer/bin/Register.pm
  cp -r -p itools/WebDNA.pm ${ITOOLS_BIN}
  cp -r -p itools/webdna.gif ${TENON_BASE_DIR}/AdminServer/image/
  IS_ITOOLS_CONF_EXIST=`grep WebDNA ${ITOOLS_REGISTER_PM}`
  if [ ."${IS_ITOOLS_CONF_EXIST}" = . ]; then
    echo "Modifying ${ITOOLS_REGISTER_PM}"
    echo "/use Mail;/
i
use WebDNA; #6.2
.
w
q" | ed -s "${ITOOLS_REGISTER_PM}" > /dev/null
  fi
fi

./post_scripts.sh ${HTML} ${ENGINE_DIR} ${HTTPD} ${WWW_USER}

###No need to thin any more
#if [ ${WD_ARCH} = "macosx" ]; then
#  #Need to thin apache to 32bit.
#  HTTPD_IS_UNIV_BINARY=`file /usr/sbin/httpd |grep -i universal`
#
#  if [ -n "${HTTPD_IS_UNIV_BINARY}" ]; then
#     HTTPD_UNIV_BINARY=/usr/sbin/httpd.webdna_backup_universal_binary
#     echo "Backing up httpd to ${HTTPD_UNIV_BINARY}"
#     sudo cp -p /usr/sbin/httpd ${HTTPD_UNIV_BINARY}
#     sudo lipo -thin i386 /usr/sbin/httpd -output /usr/sbin/httpd
#  fi
#fi


# restart apache
sudo ${APACHE_CTL} start

# start WebDNA
sudo ${ENGINE_DIR}/WebCatalogCtl start

#
# Create the files needed so that WebCatalog is 
# started after the machine is rebooted.
#
if [ ${WD_ARCH} != "macosx" ]; then
  rm -f /etc/init.d/WebCatalogCtl
  rm -f /etc/init.d/rc*.d/*WebCatalog
  ln -s ${ENGINE_DIR}/WebCatalogCtl /etc/init.d/
fi
#ln -s ../WebCatalogCtl /etc/init.d/rc3.d/K03WebCatalog
#ln -s ../WebCatalogCtl /etc/init.d/rc5.d/K03WebCatalog
#ln -s ../WebCatalogCtl /etc/init.d/rc3.d/S90WebCatalog
#ln -s ../WebCatalogCtl /etc/init.d/rc5.d/S90WebCatalog

$ECHO " "
$ECHO " +--------------------------------------------------------"
$ECHO " | You now have successfully installed WebDNA in your     "
$ECHO " | Apache server.                                         " 
$ECHO " | To administer your WebDNA installation, please visit   "
$ECHO " | http://127.0.0.1/WebCatalog                            "
$ECHO " |                                                        "
$ECHO " | To start or stop WebDNA, please open a terminal shell  "
$ECHO " | and use these commands:                                " 
$ECHO " | ${ENGINE_DIR}/WebCatalogCtl start  "
$ECHO " | ${ENGINE_DIR}/WebCatalogCtl stop   "
$ECHO " | ${ENGINE_DIR}/WebCatalogCtl restart"
$ECHO " | Note: Only 'killall -9' WebDNAMonitor and WebCatalog as a last resort" 
$ECHO " |   If you do, make sure to null out the ID text in /WebCatalogEngine/WebCatalog.pid before restarting" 
$ECHO " |                                                        "
$ECHO " | For WebDNA Support, email: support@webdna.us           "
$ECHO " | http://www.webdna.us                                   "
$ECHO " |                                                        "
$ECHO " | Thank you for using WebDNA, WebDNA Software            "
$ECHO " | Corporation's complete solution for eCommerce and  "
$ECHO " | dynamic web publishing.                                "
$ECHO " +--------------------------------------------------------"
$ECHO " "

